(* An interpreter for arithmetic expressions with integers and addition. *)

(* The type of the abstract syntax tree (AST),
 * that is, expresions of the language. *)
type expr =
  | Int of int
  | Add of expr * expr

(* [is_value e] is whether [e] is a syntactic value *)
let is_value : expr -> bool = function
  | Int _ -> true
  | Add _ -> false

(* A single step of evaluation: exactly 1 step *)
let rec step : expr -> expr = function
  | Int n               -> failwith "Does not step"
  | Add(Int n1, Int n2) -> Int (n1+n2)
  | Add(Int n1, e2)     -> Add(Int n1, step e2)
  | Add(e1,e2)          -> Add(step e1, e2)

(* [eval e] is the [e -->* v] judgement.  That is,
 * keep applying [step] until a value is produced.  *)
let rec eval : expr -> expr = fun e ->
  if is_value e then e
  else eval (step e)
